@@ -93,16 +93,6 @@
 #define TZNAME_MAX _POSIX_TZNAME_MAX
 #endif
 
-/* TODO - This stuff belongs in some include/bits/ file. */
-#ifndef __BCC__
-#undef CLK_TCK
-#if (TARGET_ARCH == alpha) || (TARGET_ARCH == ia64)
-#define CLK_TCK     1024
-#else
-#define CLK_TCK     100
-#endif
-#endif
-
 /* The era code is currently unfinished. */
 /*  #define ENABLE_ERA_CODE */
 
@@ -317,19 +307,32 @@ clock_t clock(void)
 
 	times(&xtms);
 	t = ((unsigned long) xtms.tms_utime) + xtms.tms_stime;
+
+#ifndef __UCLIBC_CLK_TCK_CONST
+#error __UCLIBC_CLK_TCK_CONST not defined!
+#endif
+
+#undef CLK_TCK
+#define CLK_TCK __UCLIBC_CLK_TCK_CONST
+
+#if CLK_TCK > CLOCKS_PER_SEC
+#error __UCLIBC_CLK_TCK_CONST > CLOCKS_PER_SEC!
+#elif CLK_TCK < 1
+#error __UCLIBC_CLK_TCK_CONST < 1!
+#endif
+
 #if (CLK_TCK == CLOCKS_PER_SEC)
 	return (t <= LONG_MAX) ? t : -1;
-#elif (CLK_TCK == 1) || (CLK_TCK == 10) || (CLK_TCK == 100) || (CLK_TCK == 1000)
+#elif (CLOCKS_PER_SEC % CLK_TCK) == 0
 	return (t <= (LONG_MAX / (CLOCKS_PER_SEC/CLK_TCK)))
 		? t * (CLOCKS_PER_SEC/CLK_TCK)
 		: -1;
-#elif (CLK_TCK == 1024)
+#else
 	return (t <= ((LONG_MAX / CLOCKS_PER_SEC) * CLK_TCK
 				  + ((LONG_MAX % CLOCKS_PER_SEC) * CLK_TCK) / CLOCKS_PER_SEC))
-		? ((t >> 10) * CLOCKS_PER_SEC) + (((t & 1023) * CLOCKS_PER_SEC) >> 10)
+		? (((t / CLK_TCK) * CLOCKS_PER_SEC)
+		   + (((t % CLK_TCK) * CLOCKS_PER_SEC) / CLK_TCK))
 		: -1;
-#else
-#error fix for CLK_TCK
 #endif
 }
 
@@ -1855,9 +1858,8 @@ time_t _time_mktime(struct tm *timeptr, int store_on_success)
 	}
 
 #ifdef __BCC__
-	/* TODO - check */
 	d = p[5] - 1;
-	days = -719163L + d*365 + ((d/4) - (d/100) + (d/400) + p[3] + p[7]);
+	days = -719163L + ((long)d)*365 + ((d/4) - (d/100) + (d/400) + p[3] + p[7]);
 	secs = p[0] + 60*( p[1] + 60*((long)(p[2])) )
 		+ _time_tzinfo[timeptr->tm_isdst > 0].gmt_offset;
 	if (secs < 0) {
