@@ -22,18 +22,24 @@
  *
  */
 
+#define _GNU_SOURCE
+#define _LARGEFILE64_SOURCE
 #include <features.h>
+#undef __OPTIMIZE__
+/* We absolutely do _NOT_ want interfaces silently
+ *  *  * renamed under us or very bad things will happen... */
+#ifdef __USE_FILE_OFFSET64
+# undef __USE_FILE_OFFSET64
+#endif
+
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/syscall.h>
+#include <endian.h>
+
 
 //#define __NR_exit             1
-#ifdef L__exit
-/* Do not include unistd.h, so gcc doesn't whine about 
- * _exit returning.  It really doesn't return... */
-#define __NR__exit __NR_exit
-_syscall1(void, _exit, int, status);
-#endif
+//See _exit.c
 
 //#define __NR_fork             2
 #ifdef L___libc_fork
@@ -150,23 +156,7 @@ time_t time (time_t *t)
 #endif
 
 //#define __NR_mknod            14
-#ifdef L_mknod
-#include <unistd.h>
-extern int mknod(const char *pathname, mode_t mode, dev_t dev);
-_syscall3(int, mknod, const char *, pathname, mode_t, mode, dev_t, dev);
-
-int __xmknod (int version, const char * path, mode_t mode, dev_t *dev)
-{
-	switch(version)
-	{
-		case 1:
-			return mknod (path, mode, *dev);
-		default:
-			__set_errno(EINVAL);
-			return -1;
-	}
-}
-#endif
+//See mknod.c
 
 //#define __NR_chmod            15
 #ifdef L_chmod
@@ -198,10 +188,6 @@ _syscall3(int, lchown, const char *, path, uid_t, owner, gid_t, group);
 #define __NR___libc_lseek __NR_lseek
 _syscall3(__off_t, __libc_lseek, int, fildes, __off_t, offset, int, whence);
 weak_alias(__libc_lseek, lseek)
-#ifndef __NR__llseek
-weak_alias(__libc_lseek, llseek)
-weak_alias(__libc_lseek, lseek64)
-#endif
 #endif
 
 //#define __NR_getpid           20
@@ -231,20 +217,7 @@ _syscall1(int, umount, const char *, specialfile);
 #endif
 
 //#define __NR_setuid           23
-#ifdef L___setuid
-#define __NR___setuid __NR_setuid
-#include <unistd.h>
-static inline 
-_syscall1(int, __setuid, uid_t, uid);
-int setuid(uid_t uid)
-{
-	if (uid == (uid_t) ~0) {
-		__set_errno (EINVAL);
-		return -1;
-	}
-	return(__setuid(uid));
-}
-#endif
+// See setuid.c
 
 //#define __NR_getuid           24
 #ifdef L_getuid
@@ -422,10 +395,7 @@ int nice (int incr)
 //#define __NR_ftime            35
 
 //#define __NR_sync             36
-#ifdef L_sync
-#include <unistd.h>
-_syscall0(void, sync);
-#endif
+//See sync.c
 
 //#define __NR_kill             37
 #ifdef L_kill
@@ -723,8 +693,8 @@ int setrlimit (__rlimit_resource_t resource, const struct rlimit *rlimits)
 #else /* We don't need to wrap setrlimit */
 #ifdef L_setrlimit
 #include <unistd.h>
-#include <sys/resource.h>
-_syscall2(int, setrlimit, int, resource, const struct rlimit *, rlim);
+struct rlimit;
+_syscall2(int, setrlimit, unsigned int, resource, const struct rlimit *, rlim);
 #endif
 #endif /* __NR_setrlimit */
 
@@ -936,9 +906,10 @@ _syscall2(int, fstatfs, int, fd, struct statfs *, buf);
 #endif
 
 //#define __NR_socketcall       102
-#ifdef L_socketcall
+#ifdef L___socketcall
 #ifdef __NR_socketcall
-_syscall2(int, socketcall, int, call, unsigned long *, args);
+#define __NR___socketcall __NR_socketcall
+_syscall2(int, __socketcall, int, call, unsigned long *, args);
 #endif
 #endif
 
@@ -984,7 +955,7 @@ int __xstat(int version, const char * file_name, struct libc_stat * cstat)
 	int result = __stat(file_name, &kstat);
 
 	if (result == 0) { 
-		statfix(cstat, &kstat);
+		__statfix(cstat, &kstat);
 	}
 	return result;
 }
@@ -1013,7 +984,7 @@ int __lxstat(int version, const char * file_name, struct libc_stat * cstat)
 	int result = __lstat(file_name, &kstat);
 
 	if (result == 0) { 
-		statfix(cstat, &kstat);
+		__statfix(cstat, &kstat);
 	}
 	return result;
 }
@@ -1042,7 +1013,7 @@ int __fxstat(int version, int fd, struct libc_stat * cstat)
 	int result = __fstat(fd, &kstat);
 
 	if (result == 0) { 
-		statfix(cstat, &kstat);
+		__statfix(cstat, &kstat);
 	}
 	return result;
 }
@@ -1239,30 +1210,7 @@ _syscall1(int, setfsgid, gid_t, gid);
 #endif
 
 //#define __NR__llseek          140
-#ifdef L__llseek
-#ifdef __UCLIBC_HAVE_LFS__
-#ifdef __NR__llseek
-extern int _llseek(int fd, __off_t offset_hi, __off_t offset_lo, 
-		__loff_t *result, int whence);
-
-_syscall5(int, _llseek, int, fd, __off_t, offset_hi, __off_t, offset_lo, 
-		__loff_t *, result, int, whence);
-
-__loff_t __libc_lseek64(int fd, __loff_t offset, int whence)
-{
-	int ret;
-	__loff_t result;
-
-	ret = _llseek(fd, (__off_t) (offset >> 32),
-				  (__off_t) (offset & 0xffffffff), &result, whence);
-
-	return ret ? (__loff_t) ret : result;
-}
-weak_alias(__libc_lseek64, llseek);
-weak_alias(__libc_lseek64, lseek64);
-#endif
-#endif
-#endif
+//See llseek.c
 
 //#define __NR_getdents         141
 #ifdef L_getdents
@@ -1517,11 +1465,11 @@ int sigpending(sigset_t *set)
 
 //#define __NR_rt_sigtimedwait          177
 #ifdef L___rt_sigtimedwait
-#ifdef __NR_rt_sigtimedwait
-#define __NR___rt_sigtimedwait __NR_rt_sigtimedwait
 #include <signal.h>
 #define __need_NULL
 #include <stddef.h>
+#ifdef __NR_rt_sigtimedwait
+#define __NR___rt_sigtimedwait __NR_rt_sigtimedwait
 _syscall4(int, __rt_sigtimedwait, const sigset_t *, set, siginfo_t *, info, 
 		const struct timespec *, timeout, size_t, setsize);
 
@@ -1573,38 +1521,10 @@ int sigsuspend (const sigset_t *mask)
 #endif
 
 //#define __NR_pread                    180
-#ifdef L___libc_pread
-#ifdef __NR_pread
-#define _XOPEN_SOURCE 500
-#include <unistd.h>
-#define __NR___libc_pread __NR_pread
-_syscall4(ssize_t, __libc_pread, int, fd, void *, buf, size_t, count, __off_t, offset);
-weak_alias (__libc_pread, pread)
-#else
-ssize_t pread(int fd, void *buf, size_t count, off_t offset)                                                
-{                                                                                                           
-	__set_errno(ENOSYS);                                                                                    
-	return -1;                                                                                              
-}                                                                                                           
-#endif
-#endif
+// See pread_write.c
 
 //#define __NR_pwrite                   181
-#ifdef L___libc_pwrite
-#ifdef __NR_pwrite
-#define _XOPEN_SOURCE 500
-#include <unistd.h>
-#define __NR___libc_pwrite __NR_pwrite
-_syscall4(ssize_t, __libc_pwrite, int, fd, const void *, buf, size_t, count, __off_t, offset);
-weak_alias (__libc_pwrite, pwrite)
-#else
-ssize_t pwrite(int fd, const void *buf, size_t count, off_t offset)                                         
-{                                                                                                           
-	__set_errno(ENOSYS);                                                                                    
-	return -1;                                                                                              
-}                                                                                                           
-#endif
-#endif
+// See pread_write.c
 
 //#define __NR_chown                    182
 #ifdef L_chown
@@ -1705,7 +1625,7 @@ int __xstat64(int version, const char * file_name, struct libc_stat64 * cstat)
 	int result = __stat64(file_name, &kstat);
 
 	if (result == 0) { 
-		statfix64(cstat, &kstat);
+		__statfix64(cstat, &kstat);
 	}
 	return result;
 }
@@ -1732,7 +1652,7 @@ int __lxstat64(int version, const char * file_name, struct libc_stat64 * cstat)
 	int result = __lstat64(file_name, &kstat);
 
 	if (result == 0) { 
-		statfix64(cstat, &kstat);
+		__statfix64(cstat, &kstat);
 	}
 	return result;
 }
@@ -1759,7 +1679,7 @@ int __fxstat64(int version, int fd, struct libc_stat64 * cstat)
 	int result = __fstat64(fd, &kstat);
 
 	if (result == 0) { 
-		statfix64(cstat, &kstat);
+		__statfix64(cstat, &kstat);
 	}
 	return result;
 }
